@@ -35,11 +35,16 @@
  * stored in a drm_intel_bo.
  */
 
+#include <stdbool.h>
 #include <xf86drm.h>
 
 #include "main/mtypes.h"
 #include "intel_bufmgr.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct intel_context;
 struct intel_buffer_object;
 
@@ -53,17 +58,16 @@ struct intel_buffer_object;
  */
 struct intel_region
 {
-   drm_intel_bo *buffer;  /**< buffer manager's buffer */
+   drm_intel_bo *bo;  /**< buffer manager's buffer */
    GLuint refcount; /**< Reference count for region */
    GLuint cpp;      /**< bytes per pixel */
    GLuint width;    /**< in pixels */
    GLuint height;   /**< in pixels */
-   GLuint pitch;    /**< in pixels */
+   GLuint pitch;    /**< in bytes */
    GLubyte *map;    /**< only non-NULL when region is actually mapped */
    GLuint map_refcount;  /**< Reference count for mapping */
 
    uint32_t tiling; /**< Which tiling mode the region is in */
-   struct intel_buffer_object *pbo;     /* zero-copy uploads */
 
    uint32_t name; /**< Global name for the bo */
    struct intel_screen *screen;
@@ -77,7 +81,7 @@ struct intel_region *intel_region_alloc(struct intel_screen *screen,
                                         uint32_t tiling,
 					GLuint cpp, GLuint width,
                                         GLuint height,
-					GLboolean expect_accelerated_upload);
+					bool expect_accelerated_upload);
 
 struct intel_region *
 intel_region_alloc_for_handle(struct intel_screen *screen,
@@ -85,7 +89,7 @@ intel_region_alloc_for_handle(struct intel_screen *screen,
 			      GLuint width, GLuint height, GLuint pitch,
 			      unsigned int handle, const char *name);
 
-GLboolean
+bool
 intel_region_flink(struct intel_region *region, uint32_t *name);
 
 void intel_region_reference(struct intel_region **dst,
@@ -95,26 +99,20 @@ void intel_region_release(struct intel_region **ib);
 
 void intel_recreate_static_regions(struct intel_context *intel);
 
-/* Map/unmap regions.  This is refcounted also: 
+/**
+ * Map/unmap regions.  This is refcounted also:
+ *
+ * \param mode  bitmask of GL_MAP_READ_BIT, GL_MAP_WRITE_BIT
  */
-GLubyte *intel_region_map(struct intel_context *intel,
-                          struct intel_region *ib);
+void *intel_region_map(struct intel_context *intel,
+		       struct intel_region *ib,
+		       GLbitfield mode);
 
 void intel_region_unmap(struct intel_context *intel, struct intel_region *ib);
 
-
-/* Upload data to a rectangular sub-region
- */
-void intel_region_data(struct intel_context *intel,
-                       struct intel_region *dest,
-                       GLuint dest_offset,
-                       GLuint destx, GLuint desty,
-                       const void *src, GLuint src_stride,
-                       GLuint srcx, GLuint srcy, GLuint width, GLuint height);
-
 /* Copy rectangular sub-regions
  */
-GLboolean
+bool
 intel_region_copy(struct intel_context *intel,
 		  struct intel_region *dest,
 		  GLuint dest_offset,
@@ -122,23 +120,9 @@ intel_region_copy(struct intel_context *intel,
 		  struct intel_region *src,
 		  GLuint src_offset,
 		  GLuint srcx, GLuint srcy, GLuint width, GLuint height,
-		  GLboolean flip,
+		  bool flip,
 		  GLenum logicop);
 
-/* Helpers for zerocopy uploads, particularly texture image uploads:
- */
-void intel_region_attach_pbo(struct intel_context *intel,
-                             struct intel_region *region,
-                             struct intel_buffer_object *pbo);
-void intel_region_release_pbo(struct intel_context *intel,
-                              struct intel_region *region);
-void intel_region_cow(struct intel_context *intel,
-                      struct intel_region *region);
-
-drm_intel_bo *intel_region_buffer(struct intel_context *intel,
-				  struct intel_region *region,
-				  GLuint flag);
-
 void _mesa_copy_rect(GLubyte * dst,
                 GLuint cpp,
                 GLuint dst_pitch,
@@ -149,12 +133,52 @@ void _mesa_copy_rect(GLubyte * dst,
                 const GLubyte * src,
                 GLuint src_pitch, GLuint src_x, GLuint src_y);
 
+void
+intel_region_get_tile_masks(struct intel_region *region,
+                            uint32_t *mask_x, uint32_t *mask_y,
+                            bool map_stencil_as_y_tiled);
+
+uint32_t
+intel_region_get_aligned_offset(struct intel_region *region, uint32_t x,
+                                uint32_t y, bool map_stencil_as_y_tiled);
+
+/**
+ * Used with images created with image_from_names
+ * to help support planar images.
+ */
+struct intel_image_format {
+   int fourcc;
+   int components;
+   int nplanes;
+   struct {
+      int buffer_index;
+      int width_shift;
+      int height_shift;
+      uint32_t dri_format;
+      int cpp;
+   } planes[3];
+};
+
 struct __DRIimageRec {
    struct intel_region *region;
    GLenum internal_format;
+   uint32_t dri_format;
    GLuint format;
-   GLenum data_type;
+   uint32_t offset;
+
+   /*
+    * Need to save these here between calls to
+    * image_from_names and calls to image_from_planar.
+    */
+   uint32_t strides[3];
+   uint32_t offsets[3];
+   struct intel_image_format *planar_format;
+
    void *data;
 };
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
