@@ -40,6 +40,8 @@
 #include "context.h"
 #include "imports.h"
 #include "macros.h"
+#include "mfeatures.h"
+#include "mtypes.h"
 #include "main/dispatch.h"
 
 typedef void (GLAPIENTRY *array_func)( const void * );
@@ -81,9 +83,6 @@ typedef struct {
 #define NUM_TYPES 8
 
 
-#if FEATURE_arrayelt
-
-
 static const int ColorFuncs[2][NUM_TYPES] = {
    {
       _gloffset_Color3bv,
@@ -173,31 +172,31 @@ static int FogCoordFuncs[NUM_TYPES];
 
 /* GL_BYTE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NbvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1bvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NbvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]), BYTE_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2bvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NbvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]),
@@ -205,13 +204,13 @@ VertexAttrib3NbvNV(GLuint index, const GLbyte *v)
 					       BYTE_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3bvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NbvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]),
@@ -220,7 +219,7 @@ VertexAttrib4NbvNV(GLuint index, const GLbyte *v)
 					       BYTE_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4bvNV(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2], (GLfloat)v[3]));
@@ -228,46 +227,46 @@ VertexAttrib4bvNV(GLuint index, const GLbyte *v)
 
 /* GL_UNSIGNED_BYTE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, UBYTE_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1ubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, UBYTE_TO_FLOAT(v[0]),
                                           UBYTE_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2ubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, UBYTE_TO_FLOAT(v[0]),
 					       UBYTE_TO_FLOAT(v[1]),
 					       UBYTE_TO_FLOAT(v[2])));
 }
-static void
+static void GLAPIENTRY
 VertexAttrib3ubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0],
                                           (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, UBYTE_TO_FLOAT(v[0]),
@@ -276,7 +275,7 @@ VertexAttrib4NubvNV(GLuint index, const GLubyte *v)
                                           UBYTE_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4ubvNV(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0],
@@ -286,32 +285,32 @@ VertexAttrib4ubvNV(GLuint index, const GLubyte *v)
 
 /* GL_SHORT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NsvNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, SHORT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1svNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NsvNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, SHORT_TO_FLOAT(v[0]),
                                           SHORT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2svNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NsvNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, SHORT_TO_FLOAT(v[0]),
@@ -319,14 +318,14 @@ VertexAttrib3NsvNV(GLuint index, const GLshort *v)
 			     SHORT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3svNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
                                           (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NsvNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, SHORT_TO_FLOAT(v[0]),
@@ -335,7 +334,7 @@ VertexAttrib4NsvNV(GLuint index, const GLshort *v)
 			     SHORT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4svNV(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -344,33 +343,33 @@ VertexAttrib4svNV(GLuint index, const GLshort *v)
 
 /* GL_UNSIGNED_SHORT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NusvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1usvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NusvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
 			     USHORT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2usvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0],
                                           (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NusvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
@@ -378,14 +377,14 @@ VertexAttrib3NusvNV(GLuint index, const GLushort *v)
 					       USHORT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3usvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
                                           (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NusvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
@@ -394,7 +393,7 @@ VertexAttrib4NusvNV(GLuint index, const GLushort *v)
 					       USHORT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4usvNV(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -403,32 +402,32 @@ VertexAttrib4usvNV(GLuint index, const GLushort *v)
 
 /* GL_INT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1ivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
 					       INT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2ivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
@@ -436,14 +435,14 @@ VertexAttrib3NivNV(GLuint index, const GLint *v)
 					       INT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3ivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
                                           (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
@@ -452,7 +451,7 @@ VertexAttrib4NivNV(GLuint index, const GLint *v)
                                           INT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4ivNV(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -461,33 +460,33 @@ VertexAttrib4ivNV(GLuint index, const GLint *v)
 
 /* GL_UNSIGNED_INT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NuivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1uivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib1fNV(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NuivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
                                           UINT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2uivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib2fNV(GET_DISPATCH(), (index, (GLfloat)v[0],
                                           (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NuivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
@@ -495,14 +494,14 @@ VertexAttrib3NuivNV(GLuint index, const GLuint *v)
 					       UINT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3uivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib3fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
                                           (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NuivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
@@ -511,7 +510,7 @@ VertexAttrib4NuivNV(GLuint index, const GLuint *v)
 					       UINT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4uivNV(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib4fNV(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -520,25 +519,25 @@ VertexAttrib4uivNV(GLuint index, const GLuint *v)
 
 /* GL_FLOAT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1fvNV(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib1fvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2fvNV(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib2fvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3fvNV(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib3fvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4fvNV(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib4fvNV(GET_DISPATCH(), (index, v));
@@ -546,25 +545,25 @@ VertexAttrib4fvNV(GLuint index, const GLfloat *v)
 
 /* GL_DOUBLE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1dvNV(GLuint index, const GLdouble *v)
 {
    CALL_VertexAttrib1dvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2dvNV(GLuint index, const GLdouble *v)
 {
    CALL_VertexAttrib2dvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3dvNV(GLuint index, const GLdouble *v)
 {
    CALL_VertexAttrib3dvNV(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4dvNV(GLuint index, const GLdouble *v)
 {
    CALL_VertexAttrib4dvNV(GET_DISPATCH(), (index, v));
@@ -678,31 +677,31 @@ static attrib_func AttribFuncsNV[2][4][NUM_TYPES] = {
 
 /* GL_BYTE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NbvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1bvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NbvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]), BYTE_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2bvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NbvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]),
@@ -710,13 +709,13 @@ VertexAttrib3NbvARB(GLuint index, const GLbyte *v)
 					       BYTE_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3bvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NbvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, BYTE_TO_FLOAT(v[0]),
@@ -725,7 +724,7 @@ VertexAttrib4NbvARB(GLuint index, const GLbyte *v)
 					       BYTE_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4bvARB(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2], (GLfloat)v[3]));
@@ -733,19 +732,19 @@ VertexAttrib4bvARB(GLuint index, const GLbyte *v)
 
 /* GL_UNSIGNED_BYTE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, UBYTE_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1ubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index,
@@ -753,14 +752,14 @@ VertexAttrib2NubvARB(GLuint index, const GLubyte *v)
                                            UBYTE_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2ubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index,
                                            (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index,
@@ -768,7 +767,7 @@ VertexAttrib3NubvARB(GLuint index, const GLubyte *v)
                                            UBYTE_TO_FLOAT(v[1]),
                                            UBYTE_TO_FLOAT(v[2])));
 }
-static void
+static void GLAPIENTRY
 VertexAttrib3ubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index,
@@ -777,7 +776,7 @@ VertexAttrib3ubvARB(GLuint index, const GLubyte *v)
                                            (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(),
@@ -788,7 +787,7 @@ VertexAttrib4NubvARB(GLuint index, const GLubyte *v)
                            UBYTE_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4ubvARB(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(),
@@ -799,19 +798,19 @@ VertexAttrib4ubvARB(GLuint index, const GLubyte *v)
 
 /* GL_SHORT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NsvARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, SHORT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1svARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NsvARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(),
@@ -819,14 +818,14 @@ VertexAttrib2NsvARB(GLuint index, const GLshort *v)
                            SHORT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2svARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(),
                           (index, (GLfloat)v[0], (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NsvARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(),
@@ -836,7 +835,7 @@ VertexAttrib3NsvARB(GLuint index, const GLshort *v)
                            SHORT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3svARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(),
@@ -844,7 +843,7 @@ VertexAttrib3svARB(GLuint index, const GLshort *v)
                            (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NsvARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(),
@@ -855,7 +854,7 @@ VertexAttrib4NsvARB(GLuint index, const GLshort *v)
                            SHORT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4svARB(GLuint index, const GLshort *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -864,33 +863,33 @@ VertexAttrib4svARB(GLuint index, const GLshort *v)
 
 /* GL_UNSIGNED_SHORT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NusvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1usvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NusvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
 			     USHORT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2usvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NusvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
@@ -898,14 +897,14 @@ VertexAttrib3NusvARB(GLuint index, const GLushort *v)
 					       USHORT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3usvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NusvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, USHORT_TO_FLOAT(v[0]),
@@ -914,7 +913,7 @@ VertexAttrib4NusvARB(GLuint index, const GLushort *v)
 					       USHORT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4usvARB(GLuint index, const GLushort *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1], (GLfloat)v[2], (GLfloat)v[3]));
@@ -922,33 +921,33 @@ VertexAttrib4usvARB(GLuint index, const GLushort *v)
 
 /* GL_INT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1ivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
 					       INT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2ivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
@@ -956,14 +955,14 @@ VertexAttrib3NivARB(GLuint index, const GLint *v)
 					       INT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3ivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, INT_TO_FLOAT(v[0]),
@@ -972,7 +971,7 @@ VertexAttrib4NivARB(GLuint index, const GLint *v)
 					       INT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4ivARB(GLuint index, const GLint *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -981,33 +980,33 @@ VertexAttrib4ivARB(GLuint index, const GLint *v)
 
 /* GL_UNSIGNED_INT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1NuivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib1uivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib1fARB(GET_DISPATCH(), (index, (GLfloat)v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2NuivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
                                            UINT_TO_FLOAT(v[1])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2uivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib2fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3NuivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
@@ -1015,14 +1014,14 @@ VertexAttrib3NuivARB(GLuint index, const GLuint *v)
                                            UINT_TO_FLOAT(v[2])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3uivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib3fARB(GET_DISPATCH(), (index, (GLfloat)v[0],
                                            (GLfloat)v[1], (GLfloat)v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4NuivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, UINT_TO_FLOAT(v[0]),
@@ -1031,7 +1030,7 @@ VertexAttrib4NuivARB(GLuint index, const GLuint *v)
                                            UINT_TO_FLOAT(v[3])));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4uivARB(GLuint index, const GLuint *v)
 {
    CALL_VertexAttrib4fARB(GET_DISPATCH(), (index, (GLfloat)v[0], (GLfloat)v[1],
@@ -1040,25 +1039,25 @@ VertexAttrib4uivARB(GLuint index, const GLuint *v)
 
 /* GL_FLOAT attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1fvARB(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib1fvARB(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2fvARB(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib2fvARB(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3fvARB(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib3fvARB(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4fvARB(GLuint index, const GLfloat *v)
 {
    CALL_VertexAttrib4fvARB(GET_DISPATCH(), (index, v));
@@ -1066,180 +1065,180 @@ VertexAttrib4fvARB(GLuint index, const GLfloat *v)
 
 /* GL_DOUBLE attributes */
 
-static void
+static void GLAPIENTRY
 VertexAttrib1dvARB(GLuint index, const GLdouble *v)
 {
-   CALL_VertexAttrib1dvARB(GET_DISPATCH(), (index, v));
+   CALL_VertexAttrib1dv(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib2dvARB(GLuint index, const GLdouble *v)
 {
-   CALL_VertexAttrib2dvARB(GET_DISPATCH(), (index, v));
+   CALL_VertexAttrib2dv(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib3dvARB(GLuint index, const GLdouble *v)
 {
-   CALL_VertexAttrib3dvARB(GET_DISPATCH(), (index, v));
+   CALL_VertexAttrib3dv(GET_DISPATCH(), (index, v));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttrib4dvARB(GLuint index, const GLdouble *v)
 {
-   CALL_VertexAttrib4dvARB(GET_DISPATCH(), (index, v));
+   CALL_VertexAttrib4dv(GET_DISPATCH(), (index, v));
 }
 
 
 /**
  * Integer-valued attributes
  */
-static void
+static void GLAPIENTRY
 VertexAttribI1bv(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttribI1iEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2bv(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttribI2iEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3bv(GLuint index, const GLbyte *v)
 {
    CALL_VertexAttribI3iEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4bv(GLuint index, const GLbyte *v)
 {
-   CALL_VertexAttribI4bvEXT(GET_DISPATCH(), (index, v));
+   CALL_VertexAttribI4bv(GET_DISPATCH(), (index, v));
 }
 
 
-static void
+static void GLAPIENTRY
 VertexAttribI1ubv(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttribI1uiEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2ubv(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttribI2uiEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3ubv(GLuint index, const GLubyte *v)
 {
    CALL_VertexAttribI3uiEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4ubv(GLuint index, const GLubyte *v)
 {
-   CALL_VertexAttribI4ubvEXT(GET_DISPATCH(), (index, v));
+   CALL_VertexAttribI4ubv(GET_DISPATCH(), (index, v));
 }
 
 
 
-static void
+static void GLAPIENTRY
 VertexAttribI1sv(GLuint index, const GLshort *v)
 {
    CALL_VertexAttribI1iEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2sv(GLuint index, const GLshort *v)
 {
    CALL_VertexAttribI2iEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3sv(GLuint index, const GLshort *v)
 {
    CALL_VertexAttribI3iEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4sv(GLuint index, const GLshort *v)
 {
-   CALL_VertexAttribI4svEXT(GET_DISPATCH(), (index, v));
+   CALL_VertexAttribI4sv(GET_DISPATCH(), (index, v));
 }
 
 
-static void
+static void GLAPIENTRY
 VertexAttribI1usv(GLuint index, const GLushort *v)
 {
    CALL_VertexAttribI1uiEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2usv(GLuint index, const GLushort *v)
 {
    CALL_VertexAttribI2uiEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3usv(GLuint index, const GLushort *v)
 {
    CALL_VertexAttribI3uiEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4usv(GLuint index, const GLushort *v)
 {
-   CALL_VertexAttribI4usvEXT(GET_DISPATCH(), (index, v));
+   CALL_VertexAttribI4usv(GET_DISPATCH(), (index, v));
 }
 
 
 
-static void
+static void GLAPIENTRY
 VertexAttribI1iv(GLuint index, const GLint *v)
 {
    CALL_VertexAttribI1iEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2iv(GLuint index, const GLint *v)
 {
    CALL_VertexAttribI2iEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3iv(GLuint index, const GLint *v)
 {
    CALL_VertexAttribI3iEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4iv(GLuint index, const GLint *v)
 {
    CALL_VertexAttribI4ivEXT(GET_DISPATCH(), (index, v));
 }
 
 
-static void
+static void GLAPIENTRY
 VertexAttribI1uiv(GLuint index, const GLuint *v)
 {
    CALL_VertexAttribI1uiEXT(GET_DISPATCH(), (index, v[0]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI2uiv(GLuint index, const GLuint *v)
 {
    CALL_VertexAttribI2uiEXT(GET_DISPATCH(), (index, v[0], v[1]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI3uiv(GLuint index, const GLuint *v)
 {
    CALL_VertexAttribI3uiEXT(GET_DISPATCH(), (index, v[0], v[1], v[2]));
 }
 
-static void
+static void GLAPIENTRY
 VertexAttribI4uiv(GLuint index, const GLuint *v)
 {
    CALL_VertexAttribI4uivEXT(GET_DISPATCH(), (index, v));
@@ -1406,14 +1405,14 @@ GLboolean _ae_create_context( struct gl_context *ctx )
       return GL_TRUE;
 
    /* These _gloffset_* values may not be compile-time constants */
-   SecondaryColorFuncs[0] = _gloffset_SecondaryColor3bvEXT;
-   SecondaryColorFuncs[1] = _gloffset_SecondaryColor3ubvEXT;
-   SecondaryColorFuncs[2] = _gloffset_SecondaryColor3svEXT;
-   SecondaryColorFuncs[3] = _gloffset_SecondaryColor3usvEXT;
-   SecondaryColorFuncs[4] = _gloffset_SecondaryColor3ivEXT;
-   SecondaryColorFuncs[5] = _gloffset_SecondaryColor3uivEXT;
+   SecondaryColorFuncs[0] = _gloffset_SecondaryColor3bv;
+   SecondaryColorFuncs[1] = _gloffset_SecondaryColor3ubv;
+   SecondaryColorFuncs[2] = _gloffset_SecondaryColor3sv;
+   SecondaryColorFuncs[3] = _gloffset_SecondaryColor3usv;
+   SecondaryColorFuncs[4] = _gloffset_SecondaryColor3iv;
+   SecondaryColorFuncs[5] = _gloffset_SecondaryColor3uiv;
    SecondaryColorFuncs[6] = _gloffset_SecondaryColor3fvEXT;
-   SecondaryColorFuncs[7] = _gloffset_SecondaryColor3dvEXT;
+   SecondaryColorFuncs[7] = _gloffset_SecondaryColor3dv;
 
    FogCoordFuncs[0] = -1;
    FogCoordFuncs[1] = -1;
@@ -1422,9 +1421,9 @@ GLboolean _ae_create_context( struct gl_context *ctx )
    FogCoordFuncs[4] = -1;
    FogCoordFuncs[5] = -1;
    FogCoordFuncs[6] = _gloffset_FogCoordfvEXT;
-   FogCoordFuncs[7] = _gloffset_FogCoorddvEXT;
+   FogCoordFuncs[7] = _gloffset_FogCoorddv;
 
-   ctx->aelt_context = CALLOC( sizeof(AEcontext) );
+   ctx->aelt_context = calloc(1, sizeof(AEcontext));
    if (!ctx->aelt_context)
       return GL_FALSE;
 
@@ -1436,7 +1435,7 @@ GLboolean _ae_create_context( struct gl_context *ctx )
 void _ae_destroy_context( struct gl_context *ctx )
 {
    if ( AE_CONTEXT( ctx ) ) {
-      FREE( ctx->aelt_context );
+      free(ctx->aelt_context);
       ctx->aelt_context = NULL;
    }
 }
@@ -1472,44 +1471,44 @@ static void _ae_update_state( struct gl_context *ctx )
    actx->nr_vbos = 0;
 
    /* conventional vertex arrays */
-   if (arrayObj->Index.Enabled) {
-      aa->array = &arrayObj->Index;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_COLOR_INDEX].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_COLOR_INDEX];
       aa->offset = IndexFuncs[TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   if (arrayObj->EdgeFlag.Enabled) {
-      aa->array = &arrayObj->EdgeFlag;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_EDGEFLAG].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_EDGEFLAG];
       aa->offset = _gloffset_EdgeFlagv;
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   if (arrayObj->Normal.Enabled) {
-      aa->array = &arrayObj->Normal;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_NORMAL].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_NORMAL];
       aa->offset = NormalFuncs[TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   if (arrayObj->Color.Enabled) {
-      aa->array = &arrayObj->Color;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_COLOR0].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_COLOR0];
       aa->offset = ColorFuncs[aa->array->Size-3][TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   if (arrayObj->SecondaryColor.Enabled) {
-      aa->array = &arrayObj->SecondaryColor;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_COLOR1].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_COLOR1];
       aa->offset = SecondaryColorFuncs[TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   if (arrayObj->FogCoord.Enabled) {
-      aa->array = &arrayObj->FogCoord;
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_FOG].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_FOG];
       aa->offset = FogCoordFuncs[TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
    for (i = 0; i < ctx->Const.MaxTextureCoordUnits; i++) {
-      struct gl_client_array *attribArray = &arrayObj->TexCoord[i];
+      struct gl_client_array *attribArray = &arrayObj->VertexAttrib[VERT_ATTRIB_TEX(i)];
       if (attribArray->Enabled) {
          /* NOTE: we use generic glVertexAttribNV functions here.
           * If we ever remove GL_NV_vertex_program this will have to change.
@@ -1526,34 +1525,27 @@ static void _ae_update_state( struct gl_context *ctx )
    }
 
    /* generic vertex attribute arrays */   
-   for (i = 1; i < Elements(arrayObj->VertexAttrib); i++) {  /* skip zero! */
-      struct gl_client_array *attribArray = &arrayObj->VertexAttrib[i];
+   for (i = 1; i < VERT_ATTRIB_GENERIC_MAX; i++) {  /* skip zero! */
+      struct gl_client_array *attribArray = &arrayObj->VertexAttrib[VERT_ATTRIB_GENERIC(i)];
       if (attribArray->Enabled) {
+         GLint intOrNorm;
          at->array = attribArray;
          /* Note: we can't grab the _glapi_Dispatch->VertexAttrib1fvNV
           * function pointer here (for float arrays) since the pointer may
           * change from one execution of _ae_ArrayElement() to
           * the next.  Doing so caused UT to break.
           */
-         if (ctx->VertexProgram._Enabled
-             && ctx->VertexProgram.Current->IsNVProgram) {
-            at->func = AttribFuncsNV[at->array->Normalized]
-                                    [at->array->Size-1]
-                                    [TYPE_IDX(at->array->Type)];
-         }
-         else {
-            GLint intOrNorm;
-            if (at->array->Integer)
-               intOrNorm = 2;
-            else if (at->array->Normalized)
-               intOrNorm = 1;
-            else
-               intOrNorm = 0;
+         if (at->array->Integer)
+            intOrNorm = 2;
+         else if (at->array->Normalized)
+            intOrNorm = 1;
+         else
+            intOrNorm = 0;
+
+         at->func = AttribFuncsARB[intOrNorm]
+            [at->array->Size-1]
+            [TYPE_IDX(at->array->Type)];
 
-            at->func = AttribFuncsARB[intOrNorm]
-                                     [at->array->Size-1]
-                                     [TYPE_IDX(at->array->Type)];
-         }
          at->index = i;
 	 check_vbo(actx, at->array->BufferObj);
          at++;
@@ -1561,24 +1553,24 @@ static void _ae_update_state( struct gl_context *ctx )
    }
 
    /* finally, vertex position */
-   if (arrayObj->VertexAttrib[0].Enabled) {
+   if (arrayObj->VertexAttrib[VERT_ATTRIB_GENERIC0].Enabled) {
       /* Use glVertex(v) instead of glVertexAttrib(0, v) to be sure it's
        * issued as the last (provoking) attribute).
        */
-      aa->array = &arrayObj->VertexAttrib[0];
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_GENERIC0];
       assert(aa->array->Size >= 2); /* XXX fix someday? */
       aa->offset = VertexFuncs[aa->array->Size-2][TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
-   else if (arrayObj->Vertex.Enabled) {
-      aa->array = &arrayObj->Vertex;
+   else if (arrayObj->VertexAttrib[VERT_ATTRIB_POS].Enabled) {
+      aa->array = &arrayObj->VertexAttrib[VERT_ATTRIB_POS];
       aa->offset = VertexFuncs[aa->array->Size-2][TYPE_IDX(aa->array->Type)];
       check_vbo(actx, aa->array->BufferObj);
       aa++;
    }
 
-   check_vbo(actx, ctx->Array.ElementArrayBufferObj);
+   check_vbo(actx, arrayObj->ElementArrayBufferObj);
 
    ASSERT(at - actx->attribs <= VERT_ATTRIB_MAX);
    ASSERT(aa - actx->arrays < 32);
@@ -1600,10 +1592,10 @@ void _ae_map_vbos( struct gl_context *ctx )
       _ae_update_state(ctx);
 
    for (i = 0; i < actx->nr_vbos; i++)
-      ctx->Driver.MapBuffer(ctx,
-			    GL_ARRAY_BUFFER_ARB,
-			    GL_DYNAMIC_DRAW_ARB,
-			    actx->vbo[i]);
+      ctx->Driver.MapBufferRange(ctx, 0,
+				 actx->vbo[i]->Size,
+				 GL_MAP_READ_BIT,
+				 actx->vbo[i]);
 
    if (actx->nr_vbos)
       actx->mapped_vbos = GL_TRUE;
@@ -1620,9 +1612,7 @@ void _ae_unmap_vbos( struct gl_context *ctx )
    assert (!actx->NewState);
 
    for (i = 0; i < actx->nr_vbos; i++)
-      ctx->Driver.UnmapBuffer(ctx,
-			      GL_ARRAY_BUFFER_ARB,
-			      actx->vbo[i]);
+      ctx->Driver.UnmapBuffer(ctx, actx->vbo[i]);
 
    actx->mapped_vbos = GL_FALSE;
 }
@@ -1643,12 +1633,20 @@ void GLAPIENTRY _ae_ArrayElement( GLint elt )
    const struct _glapi_table * const disp = GET_DISPATCH();
    GLboolean do_map;
 
+   /* If PrimitiveRestart is enabled and the index is the RestartIndex
+    * then we call PrimitiveRestartNV and return.
+    */
+   if (ctx->Array._PrimitiveRestart && (elt == ctx->Array._RestartIndex)) {
+      CALL_PrimitiveRestartNV((struct _glapi_table *)disp, ());
+      return;
+   }
+
    if (actx->NewState) {
       assert(!actx->mapped_vbos);
       _ae_update_state( ctx );
    }
 
-   /* Determine if w need to map/unmap VBOs */
+   /* Determine if we need to map/unmap VBOs */
    do_map = actx->nr_vbos && !actx->mapped_vbos;
 
    if (do_map)
@@ -1703,6 +1701,3 @@ void _mesa_install_arrayelt_vtxfmt(struct _glapi_table *disp,
 {
    SET_ArrayElement(disp, vfmt->ArrayElement);
 }
-
-
-#endif /* FEATURE_arrayelt */
